#!/bin/perl

package SymbolData;
use strict;
use warnings;
my $debug = -2;

sub new {
    
    my ($class) = shift;
    my ($objectInfoPtr) = shift;
    my ($bigDFlag) = shift;
    my @objectInfo = @$objectInfoPtr;
    my $declaredLoc = "";
    my $definedLoc = "";
    my $finalString = "";

    # Set either declared or defined location, based on bigDflag sent
    if($bigDFlag) {
        $declaredLoc = $objectInfo[4];   
        print("declared loc for $objectInfo[1],$objectInfo[0] assigned value $declaredLoc\n") if $debug > 3;
    }else {
        $definedLoc = $objectInfo[4];    
    }

    #Catch the final string if it exists
    if(defined($objectInfo[7])) {
        $finalString = $objectInfo[7];
    }

    #print("\@objectInfo in SymbolData=@objectInfo\n") if $debug>3;
    #print("object name = $objectInfo[1] file=$objectInfo[3]\n") if $debug> 3;
    #print("systemcallflag = \"$objectInfo[6]\" name=$objectInfo[1]\n") if $debug> 3;
    #my $state;
    #my $container;
    #my $typedef;
    my $self;
keys(%{%{$self}->{_symbol_id}}) = 1;
keys(%{%{$self}->{_name}}) = 1;
keys(%{%{$self}->{_refcode}}) = 1;
keys(%{%{$self}->{_declaredLoc}}) = 1;
keys(%{%{$self}->{_prototypeLoc}}) = 1;
keys(%{%{$self}->{_type}}) = 32;
keys(%{%{$self}->{_setLocs}}) = 32;
keys(%{%{$self}->{_setLocs_withX}}) = 32;
keys(%{%{$self}->{_referenceLocs}}) = 32;
keys(%{%{$self}->{_referenceLocs_withX}}) = 32;
keys(%{%{$self}->{_addressTakenLocs}}) = 32;
keys(%{%{$self}->{_addressTakenLocs_withX}}) = 32;
keys(%{%{$self}->{_allLocs}}) = 128;                                       
keys(%{%{$self}->{_file}}) = 1;                                       
keys(%{%{$self}->{_scopeEnd}}) = 1;                                       
keys(%{%{$self}->{_callsFunctions}}) = 16;                                       
keys(%{%{$self}->{_callsFunctionsTotal}}) = 1;                                       
keys(%{%{$self}->{_bigDFlag}}) = 1;                                       
keys(%{%{$self}->{_className}}) = 1;                                       
keys(%{%{$self}->{_classMethodID}}) = 1;                                       
keys(%{%{$self}->{_finalString}}) = 1;                                       
keys(%{%{$self}->{_systemCallFlag}}) = 1;                                       
keys(%{%{$self}->{_sourceFile}}) = 1;                                       
keys(%{%{$self}->{_calledByAnotherFlag}}) = 1;                                       

     $self =  {
             _symbol_id      => $objectInfo[0],   # symbol id
             _name           => $objectInfo[1],   # variable name
             _refcode        => $objectInfo[2],   # reference code
             #_declaredLoc    => $objectInfo[4],  # line number of declaration ( variable or function)
             _declaredLoc    => $declaredLoc,     # line number of declaration ( variable or function)
            # _prototypeLoc   => "",              # line number of function prototype
             _prototypeLoc   => $definedLoc,      # line number of function prototype
             _type           => "               ",               # Type: function or variable
             _setLocs        => {},            # hash of lines where it's set
             _setLocs_withX  => {},            # hash of lines where it's child is set
             _referenceLocs  => {},            # hash of lines where it's referenced
             _referenceLocs_withX     => {},   # hash of lines where it's child is referenced
             _addressTakenLocs        => {},   # hash of lines where it's address is taken
             _addressTakenLocs_withX  => {},   # hash of lines where it's child's address is taken
            # _allLocs        => {0,1},            # hash of lines of every ref, set, address taken
             _allLocs        => {},            # hash of lines of every ref, set, address taken
             _file           => $objectInfo[3],   # file it's declared in
             #_memberof       => "",               # struct, typedef or enum it's a part of
             #_state          => $state,           # Function or Global, Member or Local Variable
             #_container      => $container,       # Member Variable's container blank for typedef
             #_typedef        => $typedef,         # Name of typedef, either this or field above will be empty
             #_scopeStart     => "",               # Starting line of scope of variable
             _scopeEnd       => "     ",               # Ending line of scope of variable
             _callsFunctions => {0,1},            # hash of function names which it calls
             _callsFunctionsTotal => 0,           # Total number of function names which it calls
             _bigDFlag       => $bigDFlag,        # Is this a 'D" or a 'd'?
             _className      => "",               # Classname where method is defined
             _classMethodID  => "            ",               # SymbolID of class method 
             _finalString    => $finalString,     # Final String of global symbols
             _systemCallFlag => $objectInfo[6],   # 'S' for system call 'N'
             _sourceFile     => $objectInfo[9],   # source file used when this object was generated
             _calledByAnotherFlag => 0,           # 0 if top of call tree, 1 if called inside another function; set in functionCalledBy()
          #   _parents        => " " x 50,        # array ptr of parents which class inherits from
           };



    bless $self,$class;


    set_allLocs($self,$objectInfo[3] . "..." . $declaredLoc,"D") if $declaredLoc;
    set_allLocs($self,$objectInfo[3] . "..." . $definedLoc,"d")  if $definedLoc;
    @objectInfo = '';
    $objectInfoPtr = '';

    return $self;
}


sub get_symbol_id       { $_[0]->{_symbol_id}; }
sub get_name            { $_[0]->{_name}; }
sub get_type            { $_[0]->{_type}; }
sub get_declaredLoc     { $_[0]->{_declaredLoc}; }
sub get_prototypeLoc    { $_[0]->{_prototypeLoc}; }
sub get_file            { $_[0]->{_file}; }
#sub get_memberof        { $_[0]->{_memberof}; }
#sub get_state           { $_[0]->{_state}; }
#sub get_container       { $_[0]->{_container}; }
#sub get_typedef         { $_[0]->{_typedef}; }
#sub get_scopeStart      { $_[0]->{_scopeStart}; }
sub get_scopeEnd        { $_[0]->{_scopeEnd}; }
sub get_callsFunctionsTotal  { $_[0]->{_callsFunctionsTotal}; }
sub get_bigDFlag        { $_[0]->{_bigDFlag}; }
sub get_className       { $_[0]->{_className}; }
sub get_classMethodID   { $_[0]->{_classMethodID}; }
sub get_finalString     { $_[0]->{_finalString}; }
sub get_systemCallFlag  { $_[0]->{_systemCallFlag}; }
sub get_sourceFile      { $_[0]->{_sourceFile}; }
sub get_parents         { $_[0]->{_parents}; }
sub get_calledByAnotherFlag  { $_[0]->{_calledByAnotherFlag}; }

sub set_symbol_id       { $_[0]->{symbol_id} = $_[1]; }
sub set_name            { $_[0]->{_name} = $_[1]; }
sub set_type            { $_[0]->{_type} = $_[1]; }
sub set_declaredLoc     { $_[0]->{_declaredLoc} = $_[1]; }
sub set_prototypeLoc    { $_[0]->{_prototypeLoc} = $_[1]; }
sub set_file            { $_[0]->{_file} = $_[1]; }
#sub set_memberof        { $_[0]->{_memberof} = $_[1]; }
#sub set_state           { $_[0]->{_state} = $_[1]; }
#sub set_container       { $_[0]->{_container} = $_[1]; }
#sub set_typedef         { $_[0]->{_typedef} = $_[1]; }
#sub set_scopeStart      { $_[0]->{_scopeStart} = $_[1]; }
sub set_scopeEnd        { $_[0]->{_scopeEnd} = $_[1]; }
sub set_bigDFlag        { $_[0]->{_bigDFlag} = $_[1]; }
sub set_className       { $_[0]->{_className} = $_[1]; }
sub set_classMethodID   { $_[0]->{_classMethodID} = $_[1]; }
sub set_finalString     { $_[0]->{_finalString} = $_[1]; }
sub set_systemCallFlag  { $_[0]->{_systemCallFlag} = $_[1]; }
sub set_sourceFile      { $_[0]->{_sourceFile} = $_[1]; }
sub set_parents         { $_[0]->{_parents} = $_[1]; }
sub set_calledByAnotherFlag  { $_[0]->{_calledByAnotherFlag} = $_[1]; }

# Store sets and refs in a hash (unique keys) so only unique values stored.   
                                        
sub set_setLocs { 
    unless ($_[0]->{_setLocs}{$_[1]}) { 
        $_[0]->{_setLocs}{$_[1]} = 1;
        set_allLocs($_[0],$_[1],"S");
    }
} 

sub set_setLocs_withX { 
    unless ($_[0]->{_setLocs_withX}{$_[1]}) { 
        $_[0]->{_setLocs_withX}{$_[1]} = 1;
        set_allLocs($_[0],$_[1],"SX");
    }
} 

sub set_referenceLocs { 
    unless ($_[0]->{_referenceLocs}{$_[1]}) { 
        $_[0]->{_referenceLocs}{$_[1]} = 1; 
        set_allLocs($_[0],$_[1],"R");
    }
} 

sub set_referenceLocs_withX { 
    unless ($_[0]->{_referenceLocs_withX}{$_[1]}) { 
        $_[0]->{_referenceLocs_withX}{$_[1]} = 1; 
        set_allLocs($_[0],$_[1],"RX");
    }
} 
                                              
sub set_addressTakenLocs { 
    unless ($_[0]->{_addressTakenLocs}{$_[1]}) { 
        $_[0]->{_addressTakenLocs}{$_[1]} = 1; 
        set_allLocs($_[0],$_[1],"A");
    }
} 
                       
sub set_addressTakenLocs_withX { 
    unless ($_[0]->{_addressTakenLocs_withX}{$_[1]}) { 
        $_[0]->{_addressTakenLocs_withX}{$_[1]} = 1; 
        set_allLocs($_[0],$_[1],"AX");
    }
} 
                       
sub set_callsFunctions { 
    unless ($_[0]->{_callsFunctions}{$_[1]}) { 
        print("set_callsFunctions: $_[1] called by ",$_[0]->get_name(),"\n") if $debug > 2;
        $_[0]->{_callsFunctions}{$_[1]} = 1; 
        $_[0]->{_callsFunctionsTotal}++;  # Add to total munber of functions called 

    }
} 
                       

sub get_setLocs {
    my $string = "";

    #foreach my $set (sort {$a <=> $b} keys (%{$_[0]->{_setLocs}})) {
    foreach my $set (keys (%{$_[0]->{_setLocs}})) {
                         
        #if($set != 0) {
        if($set ne "0") {
            $string = $string . $set . " ";
        }
    }
    print("string in get_setLocs: \"$string\"\n") if $debug > 3;
    return $string;
}

    
sub get_referenceLocs {
    my $string = "";
    #foreach my $ref (sort { $a <=> $b }  keys (%{$_[0]->{_referenceLocs}})) {
    foreach my $ref (keys (%{$_[0]->{_referenceLocs}})) {
        print("key in get_refLocs: \"$ref\"\n") if $debug > 3;
        #if($ref != 0) {
        if($ref ne "0") {
            $string = $string . $ref . ",";
        }
    }
    print("string in get_refLocs: \"$string\"\n") if $debug > 3;
    return $string;
}


sub get_addressTakenLocs {
    my $string = "";
    foreach my $ref (keys (%{$_[0]->{_addressTakenLocs}})) {
        print("key in get_addressTakenLocs: \"$ref\"\n") if $debug > 3;
        if($ref ne "0") {
            $string = $string . $ref . ",";
        }
    }
    print("string in get_addressTakenLocs: \"$string\"\n") if $debug > 3;
    return $string;
}


sub get_callsFunctions {
    my $string = "";
    foreach my $functionID (sort keys (%{$_[0]->{_callsFunctions}})) {

       # if($functionID != 0) {
        if($functionID ne "0") {
            $string .= $functionID . "\t";
        }
    }
    print("string in get_callsFunctions: \"$string\"\n") if $debug > 3;
    return $string;
}


sub set_allLocs {

    my($objRef,$line,$type) = @_;

    if($objRef->{_allLocs}{$line}) {
        unless($objRef->{_allLocs}{$line} =~ /$type/) { 
            $objRef->{_allLocs}{$line} .= $type;
        }
    }else{  
        $objRef->{_allLocs}{$line} = $type; 
    }
}


sub get_allLocs {
    my @allLocations;

    foreach my $line (sort {$a cmp $b} keys (%{$_[0]->{_allLocs}})) {

        if($line ne "0") {
            push(@allLocations,$line . "-" . $_[0]->{_allLocs}{$line});
        }
    }
    return @allLocations;
}


sub get_allLocsString {
    
    my $stringOfLocations = "";

    foreach my $line (sort {$a cmp $b} keys (%{$_[0]->{_allLocs}})) {

        if($line ne "0") {
            $stringOfLocations .= " " . $line . "-" . $_[0]->{_allLocs}{$line};
        }
    }
    return $stringOfLocations;
}


sub get_declaredLoc_with_file { 
    return ($_[0]->{_file} . "..." . $_[0]->{_declaredLoc}); 
}

return 1;
